package controllers

import (
	"net/http"
	"reflect"
	"space-of-cheng/helper"
	"space-of-cheng/models"

	"github.com/astaxie/beego"
	"github.com/yl10/kit/orm"
)

//Message api统一的返回消息格式
type Message struct {
	Code    int         `json:"code"`
	Message interface{} `json:"message"`
}

//BaseController 基础controller
type BaseController struct {
	beego.Controller
	Context *helper.Context
}

// BaseControllerInterface 项目约定的 Controller 须满足的接口
type BaseControllerInterface interface {
	Constructor()
}

//Prepare 继承beego的
func (c *BaseController) Prepare() {
	// 初始化上下文
	c.initContext()
	c.Context.Ready(nil)

	// 路由对应的实际 Controller 初始化
	c.initController()
}

// ResponseJson 自定义的JSON返回
func (c *BaseController) ResponseJson(msg interface{}, code ...int) {
	status := http.StatusOK
	if len(code) > 0 {
		status = code[0]
	}

	if status < http.StatusMultipleChoices {
		c.Context.Commit()
	} else {
		c.Context.RollBack()
	}

	c.Context.Destroy()

	if reflect.ValueOf(msg).Type().String() == "*errors.errorString" {
		err := msg.(error)
		c.Data["json"] = Message{status, err.Error()}
	} else {
		c.Data["json"] = Message{status, msg}
	}

	c.ServeJSON()
	c.StopRun()
}

// ResponseRaw 返回
func (c *BaseController) ResponseRaw(msg []byte) {
	c.Ctx.ResponseWriter.Write(msg)
	c.Context.Commit()
	c.Context.Destroy()

	c.StopRun()
}

// initContext 初始化 Context
func (c *BaseController) initContext() {
	// 分页设置
	pageSetting := map[string]int{
		"pageNum": 100,
	}
	// 初始化 orm
	o, _ := orm.NewXormPlus(models.Dao.NewSession())
	o.SetPageNavi(pageSetting)

	c.Context = helper.NewContext(o, nil)
}

// initAppController 执行当前路由请求对应的 Controller 初始化
func (c *BaseController) initController() {
	if ctrl, ok := c.AppController.(BaseControllerInterface); ok {
		ctrl.Constructor()
	}
}

// Options 解决跨域先发送 options
func (c *BaseController) Options() {
	c.ResponseJson(map[string]interface{}{
		"Status":  true,
		"Message": "ok",
	})
}
